@@ -17,10 +17,12 @@ from  six import string_types
 import time
 import re
 import json
+import random
 
 import sympy
 from sympy.core.numbers import Zero
 import numpy as np
+from numpy.linalg import norm
 import copy
 from .symmetry import create_T
 import prettytable
@@ -41,7 +43,7 @@ def get_float_vals():
 
     for val in vals[1:]:
         vals.append(-val)
-    return vals,np.array(vals,dtype=np.float)
+    return vals,np.array(vals,dtype=float)
 
 def approximate_float(f,prec,vals=None):
     if vals is None:
@@ -56,6 +58,34 @@ def approximate_float(f,prec,vals=None):
     else:
         return valss[pos[0][0]]
 
+def get_unique_vals(Y,dif_prec=0.1,zero_prec=1e-3):
+    uvals = []
+    for x in np.nditer(Y):
+        if abs(x) > zero_prec:
+            if len(uvals) == 0:
+                uvals = np.append(uvals,x)
+            else:
+                dif = min(abs(uvals-x))
+                if dif > dif_prec:
+                    uvals = np.append(uvals,x)
+    return uvals
+
+def get_unique_vals_symbolic(X, dif_prec=0.1, zero_prec=1e-3):
+    xs = list(set(re.findall(r'x[0-9]+', sympy.srepr(X))))
+
+    random.seed(1)
+    randoms = [(random.random() + i) * (5 * (i + 1)) for i in range(len(xs))]
+
+    sub = tuple([(x, randoms[i]) for i, x in enumerate(xs)])
+    Xst = X.copy()
+    Xst.subs(sub)
+    Xstn = Xst.convert2numpy()
+
+    uvals_s = get_unique_vals(Xstn, dif_prec, zero_prec)
+
+    return uvals_s
+
+
 def tensor2Y(X,Y_format='sympy',algo=3,reverse=False,td=False):
     """
     Converts the tensor into a numerical matrix form. Only possible for
@@ -388,7 +418,7 @@ class GenericTensor(object):
                 for i in range(self.dim2):
                     factor *= R_list[i][ind1[i], ind2[i]]
                 if numpy_mode:
-                    factor = np.float(factor)
+                    factor = float(factor)
                 ten_R[ind1] += factor * self[ind2]
 
         return ten_R
@@ -442,7 +472,7 @@ class GenericTensor(object):
                 ind2[i] = j
                 ind2 = tuple(ind2)
                 if numpy_mode:
-                    Gtij = np.float(Gt[ind[i], j])
+                    Gtij = float(Gt[ind[i], j])
                 else:
                     Gtij = Gt[ind[i], j]
                 out[ind] += Gtij * self[ind2]
@@ -614,15 +644,26 @@ class Tensor(GenericTensor):
                out[i,j] = self[i,j]
        return out
 
-    def subs(self,old,new='notset'):
-        for ind in self:
-            if new != 'notset':
-                self[ind] = self[ind].subs(old,new)
-            else:
-                self[ind] = self[ind].subs(old)
-        return self
+    def subs(self,old,new='notset',inplace=True):
+        if inplace:
+            #the default behavior is stupid since it modifies a tensor and also returns it
+            #however I'm afraid of changing it since it could break something:)
+            for ind in self:
+                if new != 'notset':
+                    self[ind] = self[ind].subs(old,new)
+                else:
+                    self[ind] = self[ind].subs(old)
+            return self
+        else:
+            out = self.copy0()
+            for ind in self:
+                if new != 'notset':
+                    out[ind] = self[ind].subs(old, new)
+                else:
+                    out[ind] = self[ind].subs(old)
+            return out
 
-    def pprint(self,print_format=None,latex=False,no_newline=False,ind_types=False,remove_zeros=False):
+    def pprint(self,print_format=None,latex=False,no_newline=False,ind_types=False,remove_zeros=False,ret=False):
 
         if remove_zeros:
             self.remove_zeros()
@@ -632,15 +673,21 @@ class Tensor(GenericTensor):
             vec = [self[i] for i in range(self.dim1)]
             vec = sympy.Matrix([vec])
             if latex:
-                print(sympy.latex(vec))
+                if not ret:
+                    print(sympy.latex(vec))
+                else:
+                    return sympy.latex(vec)
             else:
                 sympy.pprint(vec)
         elif self.dim2 == 2:
             if latex:
-                if no_newline:
-                    print(sympy.latex(self.mat()), end=' ')
+                if not ret:
+                    if no_newline:
+                        print(sympy.latex(self.mat()), end=' ')
+                    else:
+                        print(sympy.latex(self.mat()))
                 else:
-                    print(sympy.latex(self.mat()))
+                    return sympy.latex(self.mat())
             else:
                 sympy.pprint(self.mat())
         else:
@@ -734,7 +781,10 @@ class Tensor(GenericTensor):
                     self[ind] = self[ind].subs(symbol,0)
 
     def evalf(self,acc=15):
-        out = 0
+        out = self.copy()
+        for ind in self:
+            out[ind] = self[ind].evalf(acc)
+        return out
 
     def copy0(self):
         "returns a zero tensor with the same shape and transformation rules"
@@ -845,6 +895,99 @@ class Tensor(GenericTensor):
             out[ind_tuple] = sympy.sympify(inp['X'][ind])
         return out
 
+
+    def subs_tensor(self,Y):
+
+        def get_vars(expr,name):
+            return list(set(re.findall(r'{}[0-9]+'.format(name), sympy.srepr(expr))))
+
+        try:
+            var_name = list(self.v.keys())[0][0]
+        except:
+            var_name = 'x'
+
+        var_names = get_vars(self,var_name)
+        out = self.copy()
+        var_values = {}
+        for ind in out:
+            vars_ind = get_vars(out[ind],var_name)
+            if len(vars_ind) == 1:
+                var = vars_ind[0]
+                sol = sympy.solve(out[ind]-Y[ind],var)
+                if len(sol) == 1:
+                    var_values[var] = sol[0]
+        if len(var_values) != len(var_names):
+            print("Couldn't find variable values, only works for simple tensors!")
+            return None
+
+        for ind in out:
+            for x in var_names:
+                out[ind] = out[ind].subs(x,var_values[x])
+
+        return out
+
+    def check_calc_symmetry(self, Xc, scale, verbosity=0):
+
+        X0 = self.copy0()
+        symmetry_ok = True
+        if self == X0:
+
+            if norm(Xc) > scale * 1e-6:
+                symmetry_ok = False
+
+            if verbosity > 0:
+                print('OK') if symmetry_ok else print('Symmetry not OK!')
+
+                if not symmetry_ok:
+                    print('Symmetry tensor is zero')
+                    print('norm: ', norm(Xc))
+
+        else:
+            Xs = self.subs_tensor(Xc)
+            Xsn = Xs.convert2numpy()
+            if verbosity > 1:
+                print(Xsn)
+            if Xsn is None:
+                print('Warning couldnt convert to numpy')
+                return None
+            dif = norm(Xsn - Xc)
+
+            if dif > scale * 1e-4:
+                symmetry_ok = False
+
+            uvals_s = len(get_unique_vals_symbolic(self))
+            uvals_c = len(get_unique_vals(Xc, scale * 1e-3, scale * 1e-6))
+
+            if uvals_s != uvals_c:
+                symmetry_ok = False
+
+            if verbosity > 0:
+                print('OK') if symmetry_ok else print('Symmetry not OK!')
+
+                if not symmetry_ok:
+                    print('Symmetry error: ', dif)
+                    print('norm: ', norm(Xc))
+                    print('Unique vals symmetry: {}, calculation {}'.format(uvals_s, uvals_c))
+
+                    self.pprint()
+                    print(Xc)
+
+        return symmetry_ok
+
+    def convert2numpy(self,acc=None):
+        out = np.zeros((self.dim1,)*self.dim2)
+
+        try:
+            for ind in self:
+                out[ind] = self[ind].evalf(acc)
+        except:
+            out = None
+            print('Conversion failed, conversion is only possible for numeric tensors.')
+
+        return out
+
+
+
 class NumTensor(GenericTensor):
 
     def __init__(self,kind,dim1,dim2,name='x',ind_types=None):
@@ -897,7 +1040,7 @@ class NumTensor(GenericTensor):
 
     def __mul__(self,num):
         try:
-            num = np.float(num)
+            num = float(num)
         except:
             raise Exception('Can only multiply NumTensor by float!')
         out = self.copy0()
@@ -1004,9 +1147,9 @@ class NumTensor(GenericTensor):
             elif self.trans_type == 2:
                 R = sym.R
             if self.is_contravar(i) == 1:
-                R_list.append(np.array(R,dtype=np.float))
+                R_list.append(np.array(R,dtype=float))
             else:
-                R_list.append(np.array(R.inv().T,dtype=np.float))
+                R_list.append(np.array(R.inv().T,dtype=float))
 
         ten_R = self.copy0()
 
@@ -1040,8 +1183,8 @@ class NumTensor(GenericTensor):
             ten_T (tensor): the transformed tensor
         """
 
-        mat1 = np.array(T,dtype=np.float)
-        mat2 = np.array(T.inv().T,dtype=np.float)
+        mat1 = np.array(T,dtype=float)
+        mat2 = np.array(T.inv().T,dtype=float)
 
         if not in_place:
             ten_T = self.copy0()
